from .libraries import sundials_sunmatrixdense
from .sundials_types import *
from .sundials_matrix import *

# /*
#  * -----------------------------------------------------------------
#  * Programmer(s): Daniel Reynolds @ SMU
#  *                David Gardner @ LLNL
#  * Based on code sundials_direct.h by: Radu Serban @ LLNL
#  * -----------------------------------------------------------------
#  * SUNDIALS Copyright Start
#  * Copyright (c) 2002-2019, Lawrence Livermore National Security
#  * and Southern Methodist University.
#  * All rights reserved.
#  *
#  * See the top-level LICENSE and NOTICE files for details.
#  *
#  * SPDX-License-Identifier: BSD-3-Clause
#  * SUNDIALS Copyright End
#  * -----------------------------------------------------------------
#  * This is the header file for the dense implementation of the
#  * SUNMATRIX module, SUNMATRIX_DENSE.
#  *
#  * Notes:
#  *   - The definition of the generic SUNMatrix structure can be found
#  *     in the header file sundials_matrix.h.
#  *   - The definition of the type 'realtype' can be found in the
#  *     header file sundials_types.h, and it may be changed (at the
#  *     configuration stage) according to the user's needs.
#  *     The sundials_types.h file also contains the definition
#  *     for the type 'booleantype' and 'indextype'.
#  * -----------------------------------------------------------------
#  */
#
# #ifndef _SUNMATRIX_DENSE_H
# #define _SUNMATRIX_DENSE_H
#
# #include <stdio.h>
# #include <sundials/sundials_matrix.h>
#
# #ifdef __cplusplus  /* wrapper to enable C++ usage */
# extern "C" {
# #endif
#
# /* ----------------------------------
#  * Dense implementation of SUNMatrix
#  * ---------------------------------- */
#
# struct _SUNMatrixContent_Dense {
#   sunindextype M;
#   sunindextype N;
#   realtype *data;
#   sunindextype ldata;
#   realtype **cols;
# };
#
# typedef struct _SUNMatrixContent_Dense *SUNMatrixContent_Dense;
#
# /* ------------------------------------
#  * Macros for access to SUNMATRIX_DENSE
#  * ------------------------------------ */
#
# #define SM_CONTENT_D(A)     ( (SUNMatrixContent_Dense)(A->content) )
#
# #define SM_ROWS_D(A)        ( SM_CONTENT_D(A)->M )
#
# #define SM_COLUMNS_D(A)     ( SM_CONTENT_D(A)->N )
#
# #define SM_LDATA_D(A)       ( SM_CONTENT_D(A)->ldata )
#
# #define SM_DATA_D(A)        ( SM_CONTENT_D(A)->data )
#
# #define SM_COLS_D(A)        ( SM_CONTENT_D(A)->cols )
#
# #define SM_COLUMN_D(A,j)    ( (SM_CONTENT_D(A)->cols)[j] )
#
# #define SM_ELEMENT_D(A,i,j) ( (SM_CONTENT_D(A)->cols)[j][i] )
#
# /* ---------------------------------------
#  * Exported Functions for SUNMATRIX_DENSE
#  * --------------------------------------- */
#
# SUNDIALS_EXPORT SUNMatrix SUNDenseMatrix(sunindextype M, sunindextype N);
SUNDenseMatrix = getattr(sundials_sunmatrixdense, 'SUNDenseMatrix')
SUNDenseMatrix.argtypes = [sunindextype, sunindextype]
SUNDenseMatrix.restype = SUNMatrix
#
# SUNDIALS_EXPORT void SUNDenseMatrix_Print(SUNMatrix A, FILE* outfile);
#
# SUNDIALS_EXPORT sunindextype SUNDenseMatrix_Rows(SUNMatrix A);
# SUNDIALS_EXPORT sunindextype SUNDenseMatrix_Columns(SUNMatrix A);
# SUNDIALS_EXPORT sunindextype SUNDenseMatrix_LData(SUNMatrix A);
# SUNDIALS_EXPORT realtype* SUNDenseMatrix_Data(SUNMatrix A);
# SUNDIALS_EXPORT realtype** SUNDenseMatrix_Cols(SUNMatrix A);
# SUNDIALS_EXPORT realtype* SUNDenseMatrix_Column(SUNMatrix A, sunindextype j);
#
# SUNDIALS_EXPORT SUNMatrix_ID SUNMatGetID_Dense(SUNMatrix A);
# SUNDIALS_EXPORT SUNMatrix SUNMatClone_Dense(SUNMatrix A);
# SUNDIALS_EXPORT void SUNMatDestroy_Dense(SUNMatrix A);
# SUNDIALS_EXPORT int SUNMatZero_Dense(SUNMatrix A);
# SUNDIALS_EXPORT int SUNMatCopy_Dense(SUNMatrix A, SUNMatrix B);
# SUNDIALS_EXPORT int SUNMatScaleAdd_Dense(realtype c, SUNMatrix A, SUNMatrix B);
# SUNDIALS_EXPORT int SUNMatScaleAddI_Dense(realtype c, SUNMatrix A);
# SUNDIALS_EXPORT int SUNMatMatvec_Dense(SUNMatrix A, N_Vector x, N_Vector y);
# SUNDIALS_EXPORT int SUNMatSpace_Dense(SUNMatrix A, long int *lenrw, long int *leniw);
#
#
# #ifdef __cplusplus
# }
# #endif
#
# #endif



import argparse
import textwrap

from .simulation import simulate_ssp
from .ssd import read_ssd
from ..util import plot_result

description = """\
Simulate an SSP

Example: 
    > python -m fmpy.ssp.simulate SampleSystem.ssp

"""

parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                 description=textwrap.dedent(description))

parser.add_argument('ssp_filename', help="filename of the SSP")
parser.add_argument('--stop-time', type=float, help="stop time for the simulation")
parser.add_argument('--show-plot', action='store_true', help="plot the results")

args = parser.parse_args()

if __name__ == '__main__':

    result = simulate_ssp(args.ssp_filename, stop_time=args.stop_time)

    if args.show_plot:

        ssd = read_ssd(args.ssp_filename)
        names = []

        for connector in ssd.system.connectors:
            names.append(connector.name)

        plot_result(result=result, names=names, window_title=args.ssp_filename)



from PyQt5.QtCore import QAbstractTableModel, Qt, QModelIndex, pyqtSignal, QSortFilterProxyModel
from PyQt5.QtGui import QPixmap


class Log(QAbstractTableModel):

    currentMessageChanged = pyqtSignal(str, str)
    numberOfDebugMessagesChanged = pyqtSignal(int)
    numberOfInfoMessagesChanged = pyqtSignal(int)
    numberOfWarningMessagesChanged = pyqtSignal(int)
    numberOfErrorMessagesChanged = pyqtSignal(int)

    @staticmethod
    def severity(level):
        return ['debug', 'info', 'warning', 'error'].index(level)

    def __init__(self, parent=None):
        super(QAbstractTableModel, self).__init__(parent)
        self.messages = []
        self.currentLevel = 'debug'
        self.numberOfDebugMessages = 0
        self.numberOfInfoMessages = 0
        self.numberOfWarningMessages = 0
        self.numberOfErrorMessages = 0

    def clear(self):
        self.beginResetModel()
        self.currentLevel = 'debug'
        self.messages[:] = []
        self.endResetModel()

        self.currentMessageChanged.emit('', '')

        self.numberOfDebugMessages = 0
        self.numberOfInfoMessages = 0
        self.numberOfWarningMessages = 0
        self.numberOfErrorMessages = 0

        self.numberOfDebugMessagesChanged.emit(self.numberOfDebugMessages)
        self.numberOfInfoMessagesChanged.emit(self.numberOfInfoMessages)
        self.numberOfWarningMessagesChanged.emit(self.numberOfWarningMessages)
        self.numberOfErrorMessagesChanged.emit(self.numberOfErrorMessages)

    def columnCount(self, parent):
        return 1

    def rowCount(self, parent):
        return len(self.messages)

    def log(self, level, text):
        import re

        last = len(self.messages)
        simplified = re.sub('\s+', ' ', text).strip()
        self.beginInsertRows(QModelIndex(), last, last)
        self.messages.append((level, simplified, text))
        self.endInsertRows()

        if self.severity(level) >= self.severity(self.currentLevel):
            self.currentLevel = level
            self.currentMessageChanged.emit(level, simplified)

        if level == 'debug':
            self.numberOfDebugMessages += 1
            self.numberOfDebugMessagesChanged.emit(self.numberOfDebugMessages)
        elif level == 'info':
            self.numberOfInfoMessages += 1
            self.numberOfInfoMessagesChanged.emit(self.numberOfInfoMessages)
        elif level == 'warning':
            self.numberOfWarningMessages += 1
            self.numberOfWarningMessagesChanged.emit(self.numberOfWarningMessages)
        elif level == 'error':
            self.numberOfErrorMessages += 1
            self.numberOfErrorMessagesChanged.emit(self.numberOfErrorMessages)

    def data(self, index, role):

        level, simplified, text = self.messages[index.row()]

        if role == Qt.DisplayRole:
            return simplified
        elif role == Qt.ToolTipRole:
            return text
        elif role == Qt.DecorationRole:
            return QPixmap(':/icons/%s-16x16.png' % level)

        return None


class LogMessagesFilterProxyModel(QSortFilterProxyModel):

    def __init__(self, parent=None):
        super(QSortFilterProxyModel, self).__init__(parent)
        self.showDebugMessages = False
        self.showInfoMessages = True
        self.showWarningMessages = True
        self.showErrorMessages = True

    def filterAcceptsRow(self, source_row, source_parent):

        model = self.sourceModel()
        index = model.index(source_row, 0, source_parent)

        if index.isValid():

            level, _, _ = model.messages[source_row]

            if level == 'debug' and not self.showDebugMessages:
                return False
            elif level == 'info' and not self.showInfoMessages:
                return False
            elif level == 'warning' and not self.showWarningMessages:
                return False
            elif level == 'error' and not  self.showErrorMessages:
                return False

        return QSortFilterProxyModel.filterAcceptsRow(self, source_row, source_parent)

    def setShowDebugMessages(self, show):
        self.showDebugMessages = show
        self.invalidateFilter()

    def setShowInfoMessages(self, show):
        self.showInfoMessages = show
        self.invalidateFilter()

    def setShowWarningMessages(self, show):
        self.showWarningMessages = show
        self.invalidateFilter()

    def setShowErrorMessages(self, show):
        self.showErrorMessages = show
        self.invalidateFilter()



from ctypes import *

# /* -----------------------------------------------------------------
#  * Programmer(s): Radu Serban and Aaron Collier @ LLNL
#  * -----------------------------------------------------------------
#  * SUNDIALS Copyright Start
#  * Copyright (c) 2002-2019, Lawrence Livermore National Security
#  * and Southern Methodist University.
#  * All rights reserved.
#  *
#  * See the top-level LICENSE and NOTICE files for details.
#  *
#  * SPDX-License-Identifier: BSD-3-Clause
#  * SUNDIALS Copyright End
#  * -----------------------------------------------------------------
#  * This is the header file for a generic NVECTOR package.
#  * It defines the N_Vector structure (_generic_N_Vector) which
#  * contains the following fields:
#  *   - an implementation-dependent 'content' field which contains
#  *     the description and actual data of the vector
#  *   - an 'ops' filed which contains a structure listing operations
#  *     acting on such vectors
#  * -----------------------------------------------------------------
#  * This header file contains:
#  *   - enumeration constants for all SUNDIALS-defined vector types,
#  *     as well as a generic type for user-supplied vector types,
#  *   - type declarations for the _generic_N_Vector and
#  *     _generic_N_Vector_Ops structures, as well as references to
#  *     pointers to such structures (N_Vector), and
#  *   - prototypes for the vector functions which operate on
#  *     N_Vector objects.
#  * -----------------------------------------------------------------
#  * At a minimum, a particular implementation of an NVECTOR must
#  * do the following:
#  *   - specify the 'content' field of N_Vector,
#  *   - implement the operations on those N_Vector objects,
#  *   - provide a constructor routine for new N_Vector objects
#  *
#  * Additionally, an NVECTOR implementation may provide the following:
#  *   - macros to access the underlying N_Vector data
#  *   - a constructor for an array of N_Vectors
#  *   - a constructor for an empty N_Vector (i.e., a new N_Vector with
#  *     a NULL data pointer).
#  *   - a routine to print the content of an N_Vector
#  * -----------------------------------------------------------------*/
#
# #ifndef _NVECTOR_H
# #define _NVECTOR_H
#
# #include <sundials/sundials_types.h>
#
# #ifdef __cplusplus  /* wrapper to enable C++ usage */
# extern "C" {
# #endif
#
#
# /* -----------------------------------------------------------------
#  * Implemented N_Vector types
#  * ----------------------------------------------------------------- */
#
# typedef enum {
#   SUNDIALS_NVEC_SERIAL,
#   SUNDIALS_NVEC_PARALLEL,
#   SUNDIALS_NVEC_OPENMP,
#   SUNDIALS_NVEC_PTHREADS,
#   SUNDIALS_NVEC_PARHYP,
#   SUNDIALS_NVEC_PETSC,
#   SUNDIALS_NVEC_CUDA,
#   SUNDIALS_NVEC_RAJA,
#   SUNDIALS_NVEC_OPENMPDEV,
#   SUNDIALS_NVEC_TRILINOS,
#   SUNDIALS_NVEC_MANYVECTOR,
#   SUNDIALS_NVEC_MPIMANYVECTOR,
#   SUNDIALS_NVEC_MPIPLUSX,
#   SUNDIALS_NVEC_CUSTOM
# } N_Vector_ID;
#
#
# /* -----------------------------------------------------------------
#  * Generic definition of N_Vector
#  * ----------------------------------------------------------------- */
#
# /* Forward reference for pointer to N_Vector_Ops object */
# typedef _SUNDIALS_STRUCT_ _generic_N_Vector_Ops *N_Vector_Ops;
#
# /* Forward reference for pointer to N_Vector object */
# typedef _SUNDIALS_STRUCT_ _generic_N_Vector *N_Vector;
#
# /* Define array of N_Vectors */
# typedef N_Vector *N_Vector_S;
#
# /* Structure containing function pointers to vector operations  */
# struct _generic_N_Vector_Ops {
#   N_Vector_ID  (*nvgetvectorid)(N_Vector);
#   N_Vector     (*nvclone)(N_Vector);
#   N_Vector     (*nvcloneempty)(N_Vector);
#   void         (*nvdestroy)(N_Vector);
#   void         (*nvspace)(N_Vector, sunindextype *, sunindextype *);
#   realtype*    (*nvgetarraypointer)(N_Vector);
#   void         (*nvsetarraypointer)(realtype *, N_Vector);
#   void*        (*nvgetcommunicator)(N_Vector);
#   sunindextype (*nvgetlength)(N_Vector);
#
#   /* standard vector operations */
#   void        (*nvlinearsum)(realtype, N_Vector, realtype, N_Vector, N_Vector);
#   void        (*nvconst)(realtype, N_Vector);
#   void        (*nvprod)(N_Vector, N_Vector, N_Vector);
#   void        (*nvdiv)(N_Vector, N_Vector, N_Vector);
#   void        (*nvscale)(realtype, N_Vector, N_Vector);
#   void        (*nvabs)(N_Vector, N_Vector);
#   void        (*nvinv)(N_Vector, N_Vector);
#   void        (*nvaddconst)(N_Vector, realtype, N_Vector);
#   realtype    (*nvdotprod)(N_Vector, N_Vector);
#   realtype    (*nvmaxnorm)(N_Vector);
#   realtype    (*nvwrmsnorm)(N_Vector, N_Vector);
#   realtype    (*nvwrmsnormmask)(N_Vector, N_Vector, N_Vector);
#   realtype    (*nvmin)(N_Vector);
#   realtype    (*nvwl2norm)(N_Vector, N_Vector);
#   realtype    (*nvl1norm)(N_Vector);
#   void        (*nvcompare)(realtype, N_Vector, N_Vector);
#   booleantype (*nvinvtest)(N_Vector, N_Vector);
#   booleantype (*nvconstrmask)(N_Vector, N_Vector, N_Vector);
#   realtype    (*nvminquotient)(N_Vector, N_Vector);
#
#   /* fused vector operations */
#   int (*nvlinearcombination)(int, realtype*, N_Vector*, N_Vector);
#   int (*nvscaleaddmulti)(int, realtype*, N_Vector, N_Vector*, N_Vector*);
#   int (*nvdotprodmulti)(int, N_Vector, N_Vector*, realtype*);
#
#   /* vector array operations */
#   int (*nvlinearsumvectorarray)(int, realtype, N_Vector*, realtype, N_Vector*,
#                                 N_Vector*);
#   int (*nvscalevectorarray)(int, realtype*, N_Vector*, N_Vector*);
#   int (*nvconstvectorarray)(int, realtype, N_Vector*);
#   int (*nvwrmsnormvectorarray)(int, N_Vector*, N_Vector*, realtype*);
#   int (*nvwrmsnormmaskvectorarray)(int, N_Vector*, N_Vector*, N_Vector, realtype*);
#   int (*nvscaleaddmultivectorarray)(int, int, realtype*, N_Vector*, N_Vector**, N_Vector**);
#   int (*nvlinearcombinationvectorarray)(int, int, realtype*, N_Vector**, N_Vector*);
#
#   /* OPTIONAL local reduction kernels (no parallel communication) */
#   realtype (*nvdotprodlocal)(N_Vector, N_Vector);
#   realtype (*nvmaxnormlocal)(N_Vector);
#   realtype (*nvminlocal)(N_Vector);
#   realtype (*nvl1normlocal)(N_Vector);
#   booleantype (*nvinvtestlocal)(N_Vector, N_Vector);
#   booleantype (*nvconstrmasklocal)(N_Vector, N_Vector, N_Vector);
#   realtype (*nvminquotientlocal)(N_Vector, N_Vector);
#   realtype (*nvwsqrsumlocal)(N_Vector, N_Vector);
#   realtype (*nvwsqrsummasklocal)(N_Vector, N_Vector, N_Vector);
# };
#
# /* A vector is a structure with an implementation-dependent
#    'content' field, and a pointer to a structure of vector
#    operations corresponding to that implementation. */
# struct _generic_N_Vector {
#   void *content;
#   N_Vector_Ops ops;
# };
class _generic_N_Vector(Structure):
    _fields_ = [('content',               c_void_p),
                ('_generic_N_Vector_Ops', c_void_p)]
#
#
# /* -----------------------------------------------------------------
#  * Functions exported by NVECTOR module
#  * ----------------------------------------------------------------- */
#
# SUNDIALS_EXPORT N_Vector N_VNewEmpty();
# SUNDIALS_EXPORT void N_VFreeEmpty(N_Vector v);
# SUNDIALS_EXPORT int N_VCopyOps(N_Vector w, N_Vector v);
#
# SUNDIALS_EXPORT N_Vector_ID N_VGetVectorID(N_Vector w);
# SUNDIALS_EXPORT N_Vector N_VClone(N_Vector w);
# SUNDIALS_EXPORT N_Vector N_VCloneEmpty(N_Vector w);
# SUNDIALS_EXPORT void N_VDestroy(N_Vector v);
# SUNDIALS_EXPORT void N_VSpace(N_Vector v, sunindextype *lrw, sunindextype *liw);
# SUNDIALS_EXPORT realtype *N_VGetArrayPointer(N_Vector v);
# SUNDIALS_EXPORT void N_VSetArrayPointer(realtype *v_data, N_Vector v);
# SUNDIALS_EXPORT void *N_VGetCommunicator(N_Vector v);
# SUNDIALS_EXPORT sunindextype N_VGetLength(N_Vector v);
#
# /* standard vector operations */
# SUNDIALS_EXPORT void N_VLinearSum(realtype a, N_Vector x, realtype b,
#                                   N_Vector y, N_Vector z);
# SUNDIALS_EXPORT void N_VConst(realtype c, N_Vector z);
# SUNDIALS_EXPORT void N_VProd(N_Vector x, N_Vector y, N_Vector z);
# SUNDIALS_EXPORT void N_VDiv(N_Vector x, N_Vector y, N_Vector z);
# SUNDIALS_EXPORT void N_VScale(realtype c, N_Vector x, N_Vector z);
# SUNDIALS_EXPORT void N_VAbs(N_Vector x, N_Vector z);
# SUNDIALS_EXPORT void N_VInv(N_Vector x, N_Vector z);
# SUNDIALS_EXPORT void N_VAddConst(N_Vector x, realtype b, N_Vector z);
# SUNDIALS_EXPORT realtype N_VDotProd(N_Vector x, N_Vector y);
# SUNDIALS_EXPORT realtype N_VMaxNorm(N_Vector x);
# SUNDIALS_EXPORT realtype N_VWrmsNorm(N_Vector x, N_Vector w);
# SUNDIALS_EXPORT realtype N_VWrmsNormMask(N_Vector x, N_Vector w, N_Vector id);
# SUNDIALS_EXPORT realtype N_VMin(N_Vector x);
# SUNDIALS_EXPORT realtype N_VWL2Norm(N_Vector x, N_Vector w);
# SUNDIALS_EXPORT realtype N_VL1Norm(N_Vector x);
# SUNDIALS_EXPORT void N_VCompare(realtype c, N_Vector x, N_Vector z);
# SUNDIALS_EXPORT booleantype N_VInvTest(N_Vector x, N_Vector z);
# SUNDIALS_EXPORT booleantype N_VConstrMask(N_Vector c, N_Vector x, N_Vector m);
# SUNDIALS_EXPORT realtype N_VMinQuotient(N_Vector num, N_Vector denom);
#
# /* OPTIONAL fused vector operations */
# SUNDIALS_EXPORT int N_VLinearCombination(int nvec, realtype* c, N_Vector* X,
#                                          N_Vector z);
#
# SUNDIALS_EXPORT int N_VScaleAddMulti(int nvec, realtype* a, N_Vector x,
#                                      N_Vector* Y, N_Vector* Z);
#
# SUNDIALS_EXPORT int N_VDotProdMulti(int nvec, N_Vector x, N_Vector* Y,
#                                     realtype* dotprods);
#
# /* OPTIONAL vector array operations */
# SUNDIALS_EXPORT int N_VLinearSumVectorArray(int nvec,
#                                             realtype a, N_Vector* X,
#                                             realtype b, N_Vector* Y,
#                                             N_Vector* Z);
#
# SUNDIALS_EXPORT int N_VScaleVectorArray(int nvec, realtype* c, N_Vector* X,
#                                         N_Vector* Z);
#
# SUNDIALS_EXPORT int N_VConstVectorArray(int nvec, realtype c, N_Vector* Z);
#
# SUNDIALS_EXPORT int N_VWrmsNormVectorArray(int nvec, N_Vector* X, N_Vector* W,
#                                            realtype* nrm);
#
# SUNDIALS_EXPORT int N_VWrmsNormMaskVectorArray(int nvec, N_Vector* X,
#                                                N_Vector* W, N_Vector id,
#                                                realtype* nrm);
#
# SUNDIALS_EXPORT int N_VScaleAddMultiVectorArray(int nvec, int nsum,
#                                                 realtype* a, N_Vector* X,
#                                                 N_Vector** Y, N_Vector** Z);
#
# SUNDIALS_EXPORT int N_VLinearCombinationVectorArray(int nvec, int nsum,
#                                                     realtype* c, N_Vector** X,
#                                                     N_Vector* Z);
#
# /* OPTIONAL local reduction kernels (no parallel communication) */
# SUNDIALS_EXPORT realtype N_VDotProdLocal(N_Vector x, N_Vector y);
# SUNDIALS_EXPORT realtype N_VMaxNormLocal(N_Vector x);
# SUNDIALS_EXPORT realtype N_VMinLocal(N_Vector x);
# SUNDIALS_EXPORT realtype N_VL1NormLocal(N_Vector x);
# SUNDIALS_EXPORT realtype N_VWSqrSumLocal(N_Vector x, N_Vector w);
# SUNDIALS_EXPORT realtype N_VWSqrSumMaskLocal(N_Vector x, N_Vector w, N_Vector id);
# SUNDIALS_EXPORT booleantype N_VInvTestLocal(N_Vector x, N_Vector z);
# SUNDIALS_EXPORT booleantype N_VConstrMaskLocal(N_Vector c, N_Vector x, N_Vector m);
# SUNDIALS_EXPORT realtype N_VMinQuotientLocal(N_Vector num, N_Vector denom);
#
#
# /* -----------------------------------------------------------------
#  * Additional functions exported by NVECTOR module
#  * ----------------------------------------------------------------- */
#
# SUNDIALS_EXPORT N_Vector* N_VNewVectorArray(int count);
# SUNDIALS_EXPORT N_Vector* N_VCloneEmptyVectorArray(int count, N_Vector w);
# SUNDIALS_EXPORT N_Vector* N_VCloneVectorArray(int count, N_Vector w);
# SUNDIALS_EXPORT void N_VDestroyVectorArray(N_Vector* vs, int count);
#
# /* These function are really only for users of the Fortran interface */
# SUNDIALS_EXPORT N_Vector N_VGetVecAtIndexVectorArray(N_Vector* vs, int index);
# SUNDIALS_EXPORT void N_VSetVecAtIndexVectorArray(N_Vector* vs, int index, N_Vector w);
#
# #ifdef __cplusplus
# }
# #endif
#
# #endif

N_Vector = POINTER(_generic_N_Vector)
